@@ -7,9 +7,8 @@ use metaclass;
 use Carp            'confess';
 use Scalar::Util    'blessed';
 
-use Data::Dumper;
-
-our $VERSION   = '0.55';
+our $VERSION   = '0.56';
+$VERSION = eval $VERSION;
 our $AUTHORITY = 'cpan:STEVAN';
 
 use base 'Moose::Meta::Role::Application';
