@@ -11,7 +11,10 @@ use overload '""'     => sub { shift->name },   # stringify to tc name
 use Carp         'confess';
 use Scalar::Util qw(blessed refaddr);
 
-our $VERSION   = '0.55';
+use base qw(Class::MOP::Object);
+
+our $VERSION   = '0.56';
+$VERSION = eval $VERSION;
 our $AUTHORITY = 'cpan:STEVAN';
 
 __PACKAGE__->meta->add_attribute('name'       => (reader => 'name'));
@@ -57,7 +60,7 @@ __PACKAGE__->meta->add_attribute('package_defined_in' => (
 
 sub new {
     my $class = shift;
-    my $self  = $class->meta->new_object(@_);
+    my $self  = $class->_new(@_);
     $self->compile_type_constraint()
         unless $self->_has_compiled_type_constraint;
     return $self;
@@ -92,7 +95,7 @@ sub get_message {
 sub equals {
     my ( $self, $type_or_name ) = @_;
 
-    my $other = Moose::Util::TypeConstraints::find_type_constraint($type_or_name);
+    my $other = Moose::Util::TypeConstraints::find_type_constraint($type_or_name) or return;
 
     return 1 if refaddr($self) == refaddr($other);
 
@@ -115,7 +118,7 @@ sub equals {
 sub is_a_type_of {
     my ($self, $type_or_name) = @_;
 
-    my $type = Moose::Util::TypeConstraints::find_type_constraint($type_or_name);
+    my $type = Moose::Util::TypeConstraints::find_type_constraint($type_or_name) or return;
 
     ($self->equals($type) || $self->is_subtype_of($type));
 }
@@ -123,7 +126,7 @@ sub is_a_type_of {
 sub is_subtype_of {
     my ($self, $type_or_name) = @_;
 
-    my $type = Moose::Util::TypeConstraints::find_type_constraint($type_or_name);
+    my $type = Moose::Util::TypeConstraints::find_type_constraint($type_or_name) or return;
 
     my $current = $self;
 
@@ -279,13 +282,24 @@ If you wish to use features at this depth, please come to the
 
 =item B<equals ($type_name_or_object)>
 
+This checks the current type against the supplied type (only).
+Returns false if the two types are not equal. It also returns false if
+you provide the type as a name, and the type name isn't found in the
+type registry.
+
 =item B<is_a_type_of ($type_name_or_object)>
 
-This checks the current type name, and if it does not match,
-checks if it is a subtype of it.
+This checks the current type against the supplied type, or if the
+current type is a sub-type of the type name or object supplied. It
+also returns false if you provide the type as a name, and the type
+name isn't found in the type registry.
 
 =item B<is_subtype_of ($type_name_or_object)>
 
+This checks the current type is a sub-type of the type name or object
+supplied. It also returns false if you provide the type as a name, and
+the type name isn't found in the type registry.
+
 =item B<compile_type_constraint>
 
 =item B<coerce ($value)>
@@ -306,10 +320,16 @@ the C<message> will be used to construct a custom error message.
 
 =item B<name>
 
+The name of the type in the global type registry.
+
 =item B<parent>
 
+This type's parent  type.
+
 =item B<has_parent>
 
+Returns true if this type has a parent type.
+
 =item B<parents>
 
 =item B<constraint>
