@@ -6,7 +6,8 @@ use metaclass;
 
 use Moose::Util::TypeConstraints ();
 
-our $VERSION   = '0.55';
+our $VERSION   = '0.56';
+$VERSION = eval $VERSION;
 our $AUTHORITY = 'cpan:STEVAN';
 
 use base 'Moose::Meta::TypeConstraint';
@@ -20,7 +21,7 @@ sub new {
 
     $args{parent} = Moose::Util::TypeConstraints::find_type_constraint('Str');
 
-    my $self = $class->meta->new_object(%args);
+    my $self = $class->_new(\%args);
 
     $self->compile_type_constraint()
         unless $self->_has_compiled_type_constraint;
